/**
 ** ParchisView.java
 **
 ** Copyright (C) 1992, 1999, 2007, 2011 Mariano Alvarez Fernandez
 ** [e-mail: malfer at telefonica.net]
 **
 ** This file is part of the Parchis4A program.
 **
 ** Parchis4A is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** Parchis4A is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with Parchis4A.  If not, see <http://www.gnu.org/licenses/>.
 **
 **/

package com.fgrim.parchis4a;

import android.content.Context;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.drawable.Drawable;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.View;

public class ParchisView extends View {

    private int resDBoard = R.drawable.tablero1;
	private Bitmap mBoard = null;
    private Bitmap mPawn[] = new Bitmap[4];
    private Bitmap mPawnCruz[] = new Bitmap[4];
    private Bitmap mPawnCruzBig[] = new Bitmap[4];
    private Bitmap mPawnCome[] = new Bitmap[4];
    private Bitmap mPawnACasa[] = new Bitmap[4];
    private Bitmap mPawnDestino;
    private Bitmap mOrdenLLeg[] = new Bitmap[4];
    private Bitmap mAndroid[] = new Bitmap[4];
    private Bitmap mAndroidT[] = new Bitmap[4];
    private Bitmap mAndroidNx[] = new Bitmap[4];
    private Bitmap mHumano[] = new Bitmap[4];
    private Bitmap mHumanoT[] = new Bitmap[4];
    private Bitmap mPodium;
    private Paint mPaint = new Paint();
    private int mBoardSize;
    private int mPawnSize;
    private int mOrdenLLegSize;
    private int mMunecoSize;
    private int mPodiumSize;
    private int mXOffset;
    private int mYOffset;
    private PosG mCasa[][] = new PosG[4][4];
    private PosG mCorredor[][] = new PosG[68][2];
    private PosG mLlegada[][][] = new PosG[4][7][2];
    private PosG mMeta[][] = new PosG[4][4];
    private PosG mPosLLeg[] = new PosG[4];
    private PosG mPosMuneco[] = new PosG[4];
    private PosG mPosPodium;
    private PosG mPosMunecoPodium[] = new PosG[4];
    private double mScale;
    private EstadoPartida mEP = null;
    
    public boolean showMunecos = true;
    public boolean showDestino = true;
    public boolean showPlPawns = false;
    
    public boolean showPawnCruz = false;
    public boolean modoCarrera = false;
    private Carrera mCarrera;
    public boolean modoMarchaPodium = false;
    private MarchaPodium mMarchaPodium;
    
    private PosG mPosPawnJg[] = new PosG[4];
    public boolean mustCalcPawnJgPos = true;

	public ParchisView(Context context, AttributeSet attrs) {
        super(context, attrs);
   }

    public ParchisView(Context context, AttributeSet attrs, int defStyle) {
    	super(context, attrs, defStyle);
    }

    public Bitmap loadBitmap(Drawable draw, int size) {
        Bitmap bitmap = Bitmap.createBitmap(size, size, Bitmap.Config.ARGB_8888);
        Canvas canvas = new Canvas(bitmap);
        draw.setBounds(0, 0, size, size);
        draw.draw(canvas);
        return bitmap;
    }

    public void setEstadoPartida(EstadoPartida ep) {
    	mEP = ep;
    }
    
    public void setCarrera(Carrera car) {
    	mCarrera = car;
    }
    
    public void setMarchaPodium(MarchaPodium mp) {
    	mMarchaPodium = mp;
    }
    
    public void setBoardImage(String value) {
    	if (value.equals("2")) resDBoard = R.drawable.tablero2;
    	else if (value.equals("3")) resDBoard = R.drawable.tablero3;
    	else resDBoard = R.drawable.tablero1;
    	mBoard = null;
    }
    
    @Override
    protected void onSizeChanged(int w, int h, int oldw, int oldh) {
        Resources r = this.getContext().getResources();

        mBoardSize = w;
        if (mBoardSize > h) mBoardSize = h;
//        if (mBoardSize > 439) mBoardSize = 439;
        mXOffset = ((w - mBoardSize) / 2);
        mYOffset = ((h - mBoardSize) / 2);
        mScale = (mBoardSize * 1.0) / 439;
        mPawnSize = (int) (19 * mScale);
        mOrdenLLegSize = (int) (37 * mScale);
        mMunecoSize = (int) (48 * mScale);
        mPodiumSize = (int) (134 * mScale);
        mBoard = loadBitmap(r.getDrawable(resDBoard), mBoardSize);
        mPawn[0] = loadBitmap(r.getDrawable(R.drawable.ficha_amarilla), mPawnSize);
        mPawn[1] = loadBitmap(r.getDrawable(R.drawable.ficha_azul), mPawnSize);
        mPawn[2] = loadBitmap(r.getDrawable(R.drawable.ficha_roja), mPawnSize);
        mPawn[3] = loadBitmap(r.getDrawable(R.drawable.ficha_verde), mPawnSize);
        mPawnCruz[0] = loadBitmap(r.getDrawable(R.drawable.ficha_amarilla_cruz), mPawnSize);
        mPawnCruz[1] = loadBitmap(r.getDrawable(R.drawable.ficha_azul_cruz), mPawnSize);
        mPawnCruz[2] = loadBitmap(r.getDrawable(R.drawable.ficha_roja_cruz), mPawnSize);
        mPawnCruz[3] = loadBitmap(r.getDrawable(R.drawable.ficha_verde_cruz), mPawnSize);
        mPawnCruzBig[0] = loadBitmap(r.getDrawable(R.drawable.ficha_amarilla_cruz), mPawnSize*2);
        mPawnCruzBig[1] = loadBitmap(r.getDrawable(R.drawable.ficha_azul_cruz), mPawnSize*2);
        mPawnCruzBig[2] = loadBitmap(r.getDrawable(R.drawable.ficha_roja_cruz), mPawnSize*2);
        mPawnCruzBig[3] = loadBitmap(r.getDrawable(R.drawable.ficha_verde_cruz), mPawnSize*2);
        mPawnCome[0] = loadBitmap(r.getDrawable(R.drawable.ficha_amarilla_come), mPawnSize*2);
        mPawnCome[1] = loadBitmap(r.getDrawable(R.drawable.ficha_azul_come), mPawnSize*2);
        mPawnCome[2] = loadBitmap(r.getDrawable(R.drawable.ficha_roja_come), mPawnSize*2);
        mPawnCome[3] = loadBitmap(r.getDrawable(R.drawable.ficha_verde_come), mPawnSize*2);
        mPawnACasa[0] = loadBitmap(r.getDrawable(R.drawable.ficha_amarilla_acasa), mPawnSize*2);
        mPawnACasa[1] = loadBitmap(r.getDrawable(R.drawable.ficha_azul_acasa), mPawnSize*2);
        mPawnACasa[2] = loadBitmap(r.getDrawable(R.drawable.ficha_roja_acasa), mPawnSize*2);
        mPawnACasa[3] = loadBitmap(r.getDrawable(R.drawable.ficha_verde_acasa), mPawnSize*2);
        mPawnDestino = loadBitmap(r.getDrawable(R.drawable.ficha_destino3), mPawnSize*2);
        mOrdenLLeg[0] = loadBitmap(r.getDrawable(R.drawable.el1), mOrdenLLegSize);
        mOrdenLLeg[1] = loadBitmap(r.getDrawable(R.drawable.el2), mOrdenLLegSize);
        mOrdenLLeg[2] = loadBitmap(r.getDrawable(R.drawable.el3), mOrdenLLegSize);
        mOrdenLLeg[3] = loadBitmap(r.getDrawable(R.drawable.el4), mOrdenLLegSize);
        mAndroid[0] = loadBitmap(r.getDrawable(R.drawable.android_amarillo), mMunecoSize);
        mAndroid[1] = loadBitmap(r.getDrawable(R.drawable.android_azul), mMunecoSize);
        mAndroid[2] = loadBitmap(r.getDrawable(R.drawable.android_rojo), mMunecoSize);
        mAndroid[3] = loadBitmap(r.getDrawable(R.drawable.android_verde), mMunecoSize);
        mAndroidT[0] = loadBitmap(r.getDrawable(R.drawable.androidt_amarillo), mMunecoSize);
        mAndroidT[1] = loadBitmap(r.getDrawable(R.drawable.androidt_azul), mMunecoSize);
        mAndroidT[2] = loadBitmap(r.getDrawable(R.drawable.androidt_rojo), mMunecoSize);
        mAndroidT[3] = loadBitmap(r.getDrawable(R.drawable.androidt_verde), mMunecoSize);
        mAndroidNx[0] = loadBitmap(r.getDrawable(R.drawable.android_c0), mMunecoSize);
        mAndroidNx[1] = loadBitmap(r.getDrawable(R.drawable.android_c1), mMunecoSize);
        mAndroidNx[2] = loadBitmap(r.getDrawable(R.drawable.android_c2), mMunecoSize);
        mAndroidNx[3] = loadBitmap(r.getDrawable(R.drawable.android_c3), mMunecoSize);
        mHumano[0] = loadBitmap(r.getDrawable(R.drawable.persona_amarilla), mMunecoSize);
        mHumano[1] = loadBitmap(r.getDrawable(R.drawable.persona_azul), mMunecoSize);
        mHumano[2] = loadBitmap(r.getDrawable(R.drawable.persona_roja), mMunecoSize);
        mHumano[3] = loadBitmap(r.getDrawable(R.drawable.persona_verde), mMunecoSize);
        mHumanoT[0] = loadBitmap(r.getDrawable(R.drawable.personat_amarilla), mMunecoSize);
        mHumanoT[1] = loadBitmap(r.getDrawable(R.drawable.personat_azul), mMunecoSize);
        mHumanoT[2] = loadBitmap(r.getDrawable(R.drawable.personat_roja), mMunecoSize);
        mHumanoT[3] = loadBitmap(r.getDrawable(R.drawable.personat_verde), mMunecoSize);
        mPodium = loadBitmap(r.getDrawable(R.drawable.podium7), mPodiumSize);
        initGBoard();
    }

    protected void initGBoard() {
    	int i;
    	int posx, posy;
    	
    	for( i=0; i<8; i++ ){
    		posx = 426 - (i * 20);
    		mCorredor[i][0] = new PosG(posx, 198);
    		mCorredor[i][1] = new PosG(posx, 174);
    	}
    	for( i=8; i<16; i++ ){
    		posy = 173 - ((i - 8) * 20);
    		mCorredor[i][0] = new PosG(261, posy);
    		mCorredor[i][1] = new PosG(285, posy);
    	}
  		mCorredor[16][0] = new PosG(215, 33);
  		mCorredor[16][1] = new PosG(239, 33);
        for( i=17; i<25; i++ ){
    		posy = 33 + ((i - 17) * 20);
    		mCorredor[i][0] = new PosG(193, posy);
    		mCorredor[i][1] = new PosG(169, posy);
    	}
	    for( i=25; i<33; i++ ){
    		posx = 168 - ((i - 25) * 20);
    		mCorredor[i][0] = new PosG(posx, 198);
    		mCorredor[i][1] = new PosG(posx, 174);
    	}
  		mCorredor[33][0] = new PosG(28, 220);
  		mCorredor[33][1] = new PosG(28, 244);
    	for( i=34; i<42; i++ ){
    		posx = 28 + ((i - 34) * 20);
    		mCorredor[i][0] = new PosG(posx, 266);
    		mCorredor[i][1] = new PosG(posx, 290);
    	}
    	for( i=42; i<50; i++ ){
    		posy = 291 + ((i - 42) * 20);
    		mCorredor[i][0] = new PosG(193, posy);
    		mCorredor[i][1] = new PosG(169, posy);
    	}
  		mCorredor[50][0] = new PosG(215, 431);
  		mCorredor[50][1] = new PosG(239, 431);
    	for( i=51; i<59; i++ ){
    		posy = 431 - ((i - 51) * 20);
    		mCorredor[i][0] = new PosG(261, posy);
    		mCorredor[i][1] = new PosG(285, posy);
    	}
    	for( i=59; i<67; i++ ){
    		posx = 286 + ((i - 59) * 20);
    		mCorredor[i][0] = new PosG(posx, 266);
    		mCorredor[i][1] = new PosG(posx, 290);
    	}
  		mCorredor[67][0] = new PosG(426, 220);
  		mCorredor[67][1] = new PosG(426, 244);

  		mCasa[0][0] = new PosG(mCorredor[ 4][0].x, mCorredor[13][0].y, false);
    	mCasa[0][1] = new PosG(mCorredor[ 2][0].x, mCorredor[13][0].y, false);
    	mCasa[0][2] = new PosG(mCorredor[ 4][0].x, mCorredor[11][0].y, false);
    	mCasa[0][3] = new PosG(mCorredor[ 2][0].x, mCorredor[11][0].y, false);
    	mCasa[1][0] = new PosG(mCorredor[30][0].x, mCorredor[19][0].y, false);
    	mCasa[1][1] = new PosG(mCorredor[28][0].x, mCorredor[19][0].y, false);
    	mCasa[1][2] = new PosG(mCorredor[30][0].x, mCorredor[21][0].y, false);
    	mCasa[1][3] = new PosG(mCorredor[28][0].x, mCorredor[21][0].y, false);
    	mCasa[2][0] = new PosG(mCorredor[36][0].x, mCorredor[45][0].y, false);
    	mCasa[2][1] = new PosG(mCorredor[38][0].x, mCorredor[45][0].y, false);
    	mCasa[2][2] = new PosG(mCorredor[36][0].x, mCorredor[47][0].y, false);
    	mCasa[2][3] = new PosG(mCorredor[38][0].x, mCorredor[47][0].y, false);
    	mCasa[3][0] = new PosG(mCorredor[62][0].x, mCorredor[55][0].y, false);
    	mCasa[3][1] = new PosG(mCorredor[64][0].x, mCorredor[55][0].y, false);
    	mCasa[3][2] = new PosG(mCorredor[62][0].x, mCorredor[53][0].y, false);
    	mCasa[3][3] = new PosG(mCorredor[64][0].x, mCorredor[53][0].y, false);

 		for( i=0; i<7; i++ ){
    		posx = 406 - i * 20;
    		mLlegada[0][i][0] = new PosG(posx, 219);
    		mLlegada[0][i][1] = new PosG(posx, 243);
    	}
    	for( i=0; i<7; i++ ){
    		posy = 53 + i * 20;
    		mLlegada[1][i][0] = new PosG(215, posy);
    		mLlegada[1][i][1] = new PosG(239, posy);
    	}
    	for( i=0; i<7; i++ ){
    		posx = 48 + i * 20;
    		mLlegada[2][i][0] = new PosG(posx, 219);
    		mLlegada[2][i][1] = new PosG(posx, 243);
    	}
    	for( i=0; i<7; i++ ){
    		posy = 411 - i * 20;
    		mLlegada[3][i][0] = new PosG(215, posy);
    		mLlegada[3][i][1] = new PosG(239, posy);
    	}

    	int x1 = 188; int x2 = 208; int x3 = 228; int x4 = 248; int x5 = 268;
    	int y1 = 193; int y2 = 213; int y3 = 233; int y4 = 253; int y5 = 273;
    	mMeta[0][0] = new PosG(x5-2, y2);
    	mMeta[0][1] = new PosG(x5-2, y3);
    	mMeta[0][2] = new PosG(x5-2, y4);
    	mMeta[0][3] = new PosG(x4-2, y3);
    	mMeta[1][0] = new PosG(x2, y1);
    	mMeta[1][1] = new PosG(x3, y1);
    	mMeta[1][2] = new PosG(x4, y1);
    	mMeta[1][3] = new PosG(x3, y2);
    	mMeta[2][0] = new PosG(x1, y2);
    	mMeta[2][1] = new PosG(x1, y3);
    	mMeta[2][2] = new PosG(x1, y4);
    	mMeta[2][3] = new PosG(x2, y3);
    	mMeta[3][0] = new PosG(x2, y5-2);
    	mMeta[3][1] = new PosG(x3, y5-2);
    	mMeta[3][2] = new PosG(x4, y5-2);
    	mMeta[3][3] = new PosG(x3, y4-2);
    	
    	mPosLLeg[0] = new PosG(340+17, 62+22);
    	mPosLLeg[1] = new PosG(62+17, 62+22);
    	mPosLLeg[2] = new PosG(62+17, 340+22);
    	mPosLLeg[3] = new PosG(340+17, 340+22);

    	mPosMuneco[0] = new PosG(382+17, 12+22);
    	mPosMuneco[1] = new PosG(10+17, 12+22);
    	mPosMuneco[2] = new PosG(10+17, 380+22);
    	mPosMuneco[3] = new PosG(382+17, 380+22);
    	
    	mPosPodium = new PosG(153+17, 296+22);
    	mPosMunecoPodium[0] = new PosG(196+17, 296+26+22);
    	mPosMunecoPodium[1] = new PosG(152+17, 296+41+22);
    	mPosMunecoPodium[2] = new PosG(240+17, 296+49+22);
    	mPosMunecoPodium[3] = new PosG(284+17, 380+22);
    }

    private void calcPawnJgPos() {
//    	if (!showPawnCruz) return;
//    	if (mEP.njgelegida < 0) return;
//        if (mEP.gjg.njugadas < 1) return;
        for (int i=0; i<mEP.gjg.njugadas; i++) {
        	int c = mEP.turno;
        	int lugar = mEP.gjg.jg[i].origen.lugar;
        	int pos = mEP.gjg.jg[i].origen.pos;
        	int ind = mEP.gjg.jg[i].origen.ind;
            if (lugar == Posicion.CORREDOR) {
            	mPosPawnJg[i] = new PosG(mCorredor[pos][ind].x, mCorredor[pos][ind].y, false);
            } else if (lugar == Posicion.LLEGADA) {
            	mPosPawnJg[i] = new PosG(mLlegada[c][pos][ind].x, mLlegada[c][pos][ind].y, false);
            }
        }
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {
    	if (modoCarrera) return true;
    	if (!showPawnCruz) return true;
    	if (mEP.njgelegida < 0) return true;
        if (mEP.gjg.njugadas < 2) return true;
        if (mEP.tipo_jugador[mEP.turno] != DefPartida.PERSONA) return true;
        if (mustCalcPawnJgPos) {
        	calcPawnJgPos();
        	mustCalcPawnJgPos = false;
        }
        if (event.getAction() == MotionEvent.ACTION_DOWN) {
            int tocX = (int)event.getX();
            int tocY = (int)event.getY();
            for (int i=0; i<mEP.gjg.njugadas; i++) { // intento en alta resolucion
                if (tocX >= mPosPawnJg[i].x && tocX <= (mPosPawnJg[i].x+mPawnSize) &&
                    tocY >= mPosPawnJg[i].y && tocY <= (mPosPawnJg[i].y+mPawnSize)) {
                    mEP.njgelegida = i;
                    invalidate();
                    return true;
                }
            }
            for (int i=0; i<mEP.gjg.njugadas; i++) { // intento en baja resolucion
                if (tocX >= (mPosPawnJg[i].x-mPawnSize/2) && tocX <= (mPosPawnJg[i].x+mPawnSize*2) &&
                    tocY >= (mPosPawnJg[i].y-mPawnSize/2) && tocY <= (mPosPawnJg[i].y+mPawnSize*2)) {
                    mEP.njgelegida = i;
                    invalidate();
                    return true;
                }
            }
        }
        return true;
    }

    @Override
    public void onDraw(Canvas canvas) {
        super.onDraw(canvas);
        if (mBoard == null) {
        	Resources r = this.getContext().getResources();
        	mBoard = loadBitmap(r.getDrawable(resDBoard), mBoardSize);
        }
        canvas.drawBitmap(mBoard, mXOffset, mYOffset, mPaint);

        drawPawns(canvas);
        if (modoCarrera) {
        	drawCarreraPawn(canvas);
        } else if (showPawnCruz) {
            if (showPlPawns)
            	drawPlayablesPawn(canvas);
            drawDestino(canvas);
            drawSelectedPawn(canvas);
        }

        if (showMunecos) {
        	drawMunecos(canvas);
            drawPodium(canvas);
        }
        drawOrdenLLeg(canvas);
    }

    private void drawOrdenLLeg(Canvas canvas) {
    	for (int c=EstadoPartida.AMAR; c<=EstadoPartida.VERD; c++) {
    		int k = mEP.jgha_terminado[c];
    		if (k < 0) continue;
            canvas.drawBitmap(mOrdenLLeg[k], mPosLLeg[c].x, mPosLLeg[c].y, mPaint);
    	}
    }

    private void draw1Muneco(Canvas canvas, int x, int y, int c, boolean blev) {
    	if (mEP.tipo_jugador[c] == DefPartida.MAQUINA) {
    		if (blev)
                canvas.drawBitmap(mAndroidT[c], x, y, mPaint);
    		else 
                canvas.drawBitmap(mAndroid[c], x, y, mPaint);
			canvas.drawBitmap(mAndroidNx[mEP.nivel_maquina[c]], x, y, mPaint);
    	} else if (mEP.tipo_jugador[c] == DefPartida.PERSONA) {
    		if (blev)
                canvas.drawBitmap(mHumanoT[c], x, y, mPaint);
    		else
                canvas.drawBitmap(mHumano[c], x, y, mPaint);
    	}
    }
    
    private void drawMunecos(Canvas canvas) {
    	if (mEP.haTerminado()) return;
    	for (int c=EstadoPartida.AMAR; c<=EstadoPartida.VERD; c++) {
    		boolean blev = (mEP.jgha_terminado[c] < 0 && c == mEP.turno);
    		draw1Muneco(canvas, mPosMuneco[c].x, mPosMuneco[c].y, c, blev);
    	}
    }

/*    private void drawMunecos(Canvas canvas) {
    	if (mEP.haTerminado()) return;
    	for (int c=EstadoPartida.AMAR; c<=EstadoPartida.VERD; c++) {
    		if (mEP.tipo_jugador[c] == DefPartida.MAQUINA) {
      		    if (mEP.ha_terminado[c] < 0 && c == mEP.turno)
                    canvas.drawBitmap(mAndroidT[c], mPosMuneco[c].x, mPosMuneco[c].y, mPaint);
    		    else
                    canvas.drawBitmap(mAndroid[c], mPosMuneco[c].x, mPosMuneco[c].y, mPaint);
    		} else if (mEP.tipo_jugador[c] == DefPartida.PERSONA) {
     		    if (mEP.ha_terminado[c] < 0 && c == mEP.turno)
     		    	 canvas.drawBitmap(mHumanoT[c], mPosMuneco[c].x, mPosMuneco[c].y, mPaint);
     		    else
    		    	 canvas.drawBitmap(mHumano[c], mPosMuneco[c].x, mPosMuneco[c].y, mPaint);
    		}
    	}
    }*/

    private void drawPawns(Canvas canvas) {
    	for (int c=EstadoPartida.AMAR; c<=EstadoPartida.VERD; c++) {
    		if (mEP.tipo_jugador[c] == DefPartida.NOJUEGA) continue;
			for (int nf=0; nf<4; nf++) {
				if (modoCarrera && c==mCarrera.cjugador && nf==mCarrera.nficha) continue;
            	int ind = mEP.pos_ficha[c][nf].ind;
            	int pos = mEP.pos_ficha[c][nf].pos;
                if (mEP.pos_ficha[c][nf].lugar == Posicion.CASA) {
                	canvas.drawBitmap(mPawn[c], mCasa[c][ind].x, mCasa[c][ind].y, mPaint);
                } else if (mEP.pos_ficha[c][nf].lugar == Posicion.CORREDOR) {
                	canvas.drawBitmap(mPawn[c], mCorredor[pos][ind].x, mCorredor[pos][ind].y, mPaint);
                } else if (mEP.pos_ficha[c][nf].lugar == Posicion.LLEGADA) {
            	    canvas.drawBitmap(mPawn[c], mLlegada[c][pos][ind].x, mLlegada[c][pos][ind].y, mPaint);
                }else if (mEP.pos_ficha[c][nf].lugar == Posicion.META) {
                	canvas.drawBitmap(mPawn[c], mMeta[c][ind].x, mMeta[c][ind].y, mPaint);
               }
			}
    	}
    }

    private void drawSelectedPawn(Canvas canvas) {
    	if (mEP.njgelegida < 0) return;
    	int c = mEP.turno;

    	int corr = mPawnSize / 2;
    	int lugar = mEP.gjg.jg[mEP.njgelegida].origen.lugar;
    	int pos = mEP.gjg.jg[mEP.njgelegida].origen.pos;
    	int ind = mEP.gjg.jg[mEP.njgelegida].origen.ind;
        if (lugar == Posicion.CORREDOR) {
        	canvas.drawBitmap(mPawnCruzBig[c], mCorredor[pos][ind].x-corr, mCorredor[pos][ind].y-corr, mPaint);
        } else if (lugar == Posicion.LLEGADA) {
    	    canvas.drawBitmap(mPawnCruzBig[c], mLlegada[c][pos][ind].x-corr, mLlegada[c][pos][ind].y-corr, mPaint);
        } else if (lugar == Posicion.CASA) { /* marca la ultima porque sino queda raro el efecto
                                                grafico al correr las demas fichas, si primer 5 marca dos */
        	ind = mEP.nfcasa[c] - 1;
        	canvas.drawBitmap(mPawnCruzBig[c], mCasa[c][ind].x-corr, mCasa[c][ind].y-corr, mPaint);
        	if (!mEP.jgha_salido[c]) {
        		ind--;
            	canvas.drawBitmap(mPawnCruzBig[c], mCasa[c][ind].x-corr, mCasa[c][ind].y-corr, mPaint);
        	}
        }
   }
    
    private void drawDestino(Canvas canvas) {
    	if (!showDestino) return;
    	if (mEP.njgelegida < 0) return;

    	int c = mEP.turno;
    	int corr = mPawnSize / 2;
    	int lugar = mEP.gjg.jg[mEP.njgelegida].destino.lugar;
    	int pos = mEP.gjg.jg[mEP.njgelegida].destino.pos;
    	int ind = mEP.gjg.jg[mEP.njgelegida].destino.ind;
        if (lugar == Posicion.CORREDOR) {
        	canvas.drawBitmap(mPawnDestino, mCorredor[pos][ind].x-corr, mCorredor[pos][ind].y-corr, mPaint);
        } else if (lugar == Posicion.LLEGADA) {
    	    canvas.drawBitmap(mPawnDestino, mLlegada[c][pos][ind].x-corr, mLlegada[c][pos][ind].y-corr, mPaint);
        } else if (lugar == Posicion.META) {
        	canvas.drawBitmap(mPawnDestino, mMeta[c][ind].x-corr, mMeta[c][ind].y-corr, mPaint);
        }
   }

    private void drawPlayablesPawn(Canvas canvas) {
    	if (mEP.njgelegida < 0) return;
    	int c = mEP.turno;

    	if (mEP.tipo_jugador[c] == DefPartida.MAQUINA) return;
    	
    	for (int i=0; i<mEP.gjg.njugadas; i++) {
    		if (i == mEP.njgelegida) continue;
    	    int lugar = mEP.gjg.jg[i].origen.lugar;
    	    int pos = mEP.gjg.jg[i].origen.pos;
    	    int ind = mEP.gjg.jg[i].origen.ind;
            if (lugar == Posicion.CORREDOR) {
        	    canvas.drawBitmap(mPawnCruz[c], mCorredor[pos][ind].x, mCorredor[pos][ind].y, mPaint);
            } else if (lugar == Posicion.LLEGADA) {
    	        canvas.drawBitmap(mPawnCruz[c], mLlegada[c][pos][ind].x, mLlegada[c][pos][ind].y, mPaint);
            }
        }
    }
    
    private void drawCarreraPawn(Canvas canvas) {
    	int c = mCarrera.cjugador;
    	if (mCarrera.sigpaso < mCarrera.npasos) {
        	int lugar = mCarrera.paso[mCarrera.sigpaso].lugar;
        	int pos = mCarrera.paso[mCarrera.sigpaso].pos;
        	int ind = mCarrera.paso[mCarrera.sigpaso].ind;
            if (lugar == Posicion.CORREDOR) {
      	        canvas.drawBitmap(mPawn[c], mCorredor[pos][ind].x, mCorredor[pos][ind].y, mPaint);
            } else if (lugar == Posicion.LLEGADA) {
	            canvas.drawBitmap(mPawn[c], mLlegada[c][pos][ind].x, mLlegada[c][pos][ind].y, mPaint);
            }
            mCarrera.sigpaso++;
    	} else { // es un especial
    		if (mCarrera.especial && mCarrera.cuentaespecial%2 == 0) {
        	    int corr = mPawnSize / 2;
        	    int lugar = mCarrera.posespecial.lugar;
        	    int pos = mCarrera.posespecial.pos;
        	    int ind = mCarrera.posespecial.ind;
    		    if (mCarrera.come && lugar == Posicion.CORREDOR)
    			    canvas.drawBitmap(mPawnCome[c], mCorredor[pos][ind].x-corr,
    			    		          mCorredor[pos][ind].y-corr, mPaint);
    		    if (mCarrera.acasa && lugar == Posicion.CASA)
    			    canvas.drawBitmap(mPawnACasa[c], mCasa[c][ind].x-corr,
    			    		          mCasa[c][ind].y-corr, mPaint);
    		    if (mCarrera.tmeta && lugar == Posicion.META) {
    		    	int k = 0;
    		    	for (int aux=EstadoPartida.AMAR; aux<=EstadoPartida.VERD; aux++) {
    		    		if (mEP.jgha_terminado[aux] >= 0) k++;
    		    	}
    			    canvas.drawBitmap(mOrdenLLeg[k], mMeta[c][ind].x-corr,
    			    		          mMeta[c][ind].y-corr, mPaint);
    		    }
    		}
    		mCarrera.cuentaespecial++;
    	}
    }
    
    private void drawPodium(Canvas canvas) {
    	if (!mEP.haTerminado()) return;
        canvas.drawBitmap(mPodium, mPosPodium.x, mPosPodium.y, mPaint);

        if (modoMarchaPodium) {
        	int x, y;
        	for (int k=0; k<mMarchaPodium.njg; k++) {
        		if (mMarchaPodium.lugar[k] == MarchaPodium.MARCHANDO) {
        			x = mPosMuneco[2].x + (int)(mMarchaPodium.paso[k] * 10 * mScale + 0.5);
        			y = mPosMuneco[2].y;
        		} else {
        			x = mPosMunecoPodium[k].x;
        			y = mPosMunecoPodium[k].y;
        		}
        		draw1Muneco(canvas, x, y, mMarchaPodium.color[k], false);
        	}
            return;	
        }
        
        for (int c=EstadoPartida.AMAR; c<=EstadoPartida.VERD; c++) {
    		int k = mEP.jgha_terminado[c];
    		if (k < 0) continue;
    		draw1Muneco(canvas, mPosMunecoPodium[k].x, mPosMunecoPodium[k].y, c, (k == 0));
    	}
    }

    private class PosG {
        public int x;
        public int y;

        public PosG(int newX, int newY) {
            x = (int) (((newX - 17) * mScale) + 0.5 + mXOffset);
            y = (int) (((newY - 22) * mScale) + 0.5 + mYOffset);
        }

        public PosG(int newX, int newY, boolean conv) {
            x = newX;
            y = newY;
        }
    }
}
